-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_ColorOutlines"

-- **************************************************
-- General information about this script
-- **************************************************

DS_ColorOutlines = {}

function DS_ColorOutlines:Name()
	return "Color Outlines"
end

function DS_ColorOutlines:Version()
	return "0.1"
end

function DS_ColorOutlines:Description()
	return "Make outlines a darker version of fill"
end

function DS_ColorOutlines:Creator()
	return "dauid.com"
end

function DS_ColorOutlines:UILabel()
	return("Color Outlines")
end
-- **************************************************
-- Recurring values
-- **************************************************
DS_ColorOutlines.colorValue = 0.16
DS_ColorOutlines.onlyBlack = true
DS_ColorOutlines.allLayers = false

--************************menu
local DS_Dialog = {}

DS_Dialog.UPDATE = MOHO.MSG_BASE

function DS_Dialog:new(moho)

	local d = LM.GUI.SimpleDialog("Settings", DS_Dialog)
	local l = d:GetLayout()
	
	d.moho = moho
	
	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("How much darker (0-1): "))
			d.blackBox = LM.GUI.CheckBox("Only color black lines")
			l:AddChild(d.blackBox, LM.GUI.ALIGN_LEFT)
			d.allBox = LM.GUI.CheckBox("All layers")
			l:AddChild(d.allBox, LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.startLoop = LM.GUI.TextControl(0, "0.0000", DS_Dialog.UPDATE, LM.GUI.FIELD_FLOAT)
			d.startLoop:SetValue(DS_ColorOutlines.colorValue)
			l:AddChild(d.startLoop)
			
		l:Pop()
	l:Pop()

	return d
end

function DS_Dialog:UpdateWidgets()
	self.blackBox:SetValue(DS_ColorOutlines.onlyBlack)
	self.allBox:SetValue(DS_ColorOutlines.allLayers)
	
	--self.loopTimes:SetValue(DS_CycleOffsetAlt.LoopTimes)
end


function DS_Dialog:OnOK()
DS_ColorOutlines.colorValue = self.startLoop:Value()
DS_ColorOutlines.onlyBlack = self.blackBox:Value()
DS_ColorOutlines.allLayers = self.allBox:Value()
end
-- **************************************************
-- The guts of this script
-- **************************************************

function DS_ColorOutlines:coloring(moho)
local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local NewStrokeColor = LM.ColorVector:new_local() 
	local StrokeColor = LM.ColorVector:new_local() 
	
	for i=0, mesh:CountShapes()-1 do
	local theShape=mesh:Shape(i)
		StrokeColor=theShape.fMyStyle.fLineCol:GetValue(moho.frame)
		NewStrokeColor=theShape.fMyStyle.fFillCol:GetValue(moho.frame)
		
		if self.onlyBlack then
			if StrokeColor.r == 0 and StrokeColor.g == 0 and StrokeColor.b == 0 then
				local darker = self.colorValue
				local nr = NewStrokeColor.r - darker
				local ng = NewStrokeColor.g - darker
				local nb = NewStrokeColor.b - darker
				local na = NewStrokeColor.a
				NewStrokeColor:Set(nr, ng, nb, na)
				theShape.fMyStyle.fLineCol:SetValue(moho.frame, NewStrokeColor)
			end
		else
			local darker = self.colorValue
			local nr = NewStrokeColor.r - darker
			local ng = NewStrokeColor.g - darker
			local nb = NewStrokeColor.b - darker
			local na = NewStrokeColor.a
			NewStrokeColor:Set(nr, ng, nb, na)
			theShape.fMyStyle.fLineCol:SetValue(moho.frame, NewStrokeColor)
		end
	end
end

function DS_ColorOutlines:allTheLayers(moho, layer, top)

    local i
    local num_layers

    if not top then
		
		self:coloring(moho)
    end

    if layer.CountLayers ~= nil then
        num_layers = layer:CountLayers()
    elseif layer:IsGroupType() then
        num_layers = moho:LayerAsGroup(layer):CountLayers()
    else
        num_layers = 0
    end

    for i = 0, num_layers-1 do
		 moho:SetSelLayer(layer:Layer(i))
        self:allTheLayers(moho, layer:Layer(i), false)
    end
    
end

function DS_ColorOutlines:Run(moho)
local dlog = DS_Dialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end
	
    local layer = moho.layer
	
	moho.document:PrepMultiUndo()
	moho.document:SetDirty()

	if self.allLayers then
		self:allTheLayers(moho, moho.document, true)
	else
		self:coloring(moho)
	end
    

end
