-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_Distort"

-- **************************************************
-- General information about this script
-- **************************************************

DS_Distort = {}


function DS_Distort:Name()
	return "Distort Points"
end

function DS_Distort:Version()
	return "0.1"
end


function DS_Distort:Description()
	return "Distort points"
end


function DS_Distort:Creator()
	return "dauid.com"
end

function DS_Distort:UILabel()
	return "Distort points"
end

-- **************************************************
-- Recurring values
-- **************************************************

DS_Distort.selCount = 1

DS_Distort.handle = 'none'
DS_Distort.bottomLeft = LM.Vector2:new_local()
DS_Distort.topLeft = LM.Vector2:new_local()
DS_Distort.bottomRight = LM.Vector2:new_local()
DS_Distort.topRight = LM.Vector2:new_local()


DS_Distort.horizonPercent = {}
DS_Distort.vertPercent = {}

DS_Distort.ptlist = {}

DS_Distort.initialized = false

DS_Distort.initialPosTable = {}


-- **************************************************
-- The guts of this script
-- **************************************************

function DS_Distort:IsEnabled(moho)
	if (moho:CountPoints() > 0) then
		return true
	end
	return false
end

function DS_Distort:IsRelevant(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	return true
end

function DS_Distort:getInitialValues(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	
	self.horizonPercent = {}
	self.vertPercent = {}
	self.vertPercent = {}
	self.ptlist = {}
	
	self.numSel = moho:CountSelectedPoints()
	if (self.numSel < 2) then
		return
	end

	local min = LM.Vector2:new_local()
	local max = LM.Vector2:new_local()

	mesh:SelectedBounds(min, max)

	self.bottomLeft.x =min.x
	self.bottomLeft.y =min.y
		
	self.topLeft.x =min.x
	self.topLeft.y =max.y
		
	self.bottomRight.x =max.x
	self.bottomRight.y =min.y
		
	self.topRight.x =max.x
	self.topRight.y =max.y
	
	for i = 0, mesh:CountPoints() - 1 do
		local pt = mesh:Point(i)
		if pt.fSelected then
			self.ptlist[i] = pt
			local maxXdistance = self.topRight.x - self.topLeft.x
			local Xdistance = pt.fPos.x - self.topLeft.x
			self.horizonPercent[i] = Xdistance/maxXdistance
			
			local maxYdistance = self.bottomLeft.y - self.topLeft.y
			local Ydistance = pt.fPos.y - self.topLeft.y
			self.vertPercent[i] = Ydistance/maxYdistance
		end
	end
	moho.view:DrawMe()
end

function DS_Distort:distort(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	for k, v in pairs(self.ptlist)do
		local topPoint = LM.Vector2:new_local()
		local bottomPoint = LM.Vector2:new_local()
		local leftPoint = LM.Vector2:new_local()
		local rightPoint = LM.Vector2:new_local()
		
		local topAngle = math.atan2(self.topRight.y - self.topLeft.y, self.topRight.x - self.topLeft.x)
		local dx = self.topRight.x - self.topLeft.x
		local dy = self.topRight.y - self.topLeft.y
		local topdist = math.sqrt(dx*dx+dy*dy)
		topPoint.x = self.topLeft.x + math.cos(topAngle)*(topdist*self.horizonPercent[k])
		topPoint.y = self.topLeft.y + math.sin(topAngle)*(topdist*self.horizonPercent[k])
		
		local bottomAngle = math.atan2(self.bottomRight.y - self.bottomLeft.y, self.bottomRight.x - self.bottomLeft.x)
		local dx = self.bottomRight.x - self.bottomLeft.x
		local dy = self.bottomRight.y - self.bottomLeft.y
		local bottomdist = math.sqrt(dx*dx+dy*dy)
		bottomPoint.x = self.bottomLeft.x + math.cos(bottomAngle)*(bottomdist*self.horizonPercent[k])
		bottomPoint.y = self.bottomLeft.y + math.sin(bottomAngle)*(bottomdist*self.horizonPercent[k])
		
		local leftAngle = math.atan2(self.bottomLeft.y - self.topLeft.y, self.bottomLeft.x - self.topLeft.x)
		local dx = self.bottomLeft.x - self.topLeft.x
		local dy = self.bottomLeft.y - self.topLeft.y
		local leftdist = math.sqrt(dx*dx+dy*dy)
		leftPoint.x = self.topLeft.x + math.cos(leftAngle)*(leftdist*self.vertPercent[k])
		leftPoint.y = self.topLeft.y + math.sin(leftAngle)*(leftdist*self.vertPercent[k])
		
		local rightAngle = math.atan2(self.bottomRight.y - self.topRight.y, self.bottomRight.x - self.topRight.x)
		local dx = self.bottomRight.x - self.topRight.x
		local dy = self.bottomRight.y - self.topRight.y
		local rightdist = math.sqrt(dx*dx+dy*dy)
		rightPoint.x = self.topRight.x + math.cos(rightAngle)*(rightdist*self.vertPercent[k])
		rightPoint.y = self.topRight.y + math.sin(rightAngle)*(rightdist*self.vertPercent[k])

		local a1, a2, b1, b2, c1, c2
 
		a1= bottomPoint.y-topPoint.y;
		b1= topPoint.x-bottomPoint.x;
		c1= bottomPoint.x*topPoint.y - topPoint.x*bottomPoint.y;
		a2= rightPoint.y-leftPoint.y;
		b2= leftPoint.x-rightPoint.x;
		c2= rightPoint.x*leftPoint.y - leftPoint.x*rightPoint.y;
 
		local denom =a1*b2 - a2*b1;
   
		v.fPos.x=(b1*c2 - b2*c1)/denom;
		v.fPos.y=(a2*c1 - a1*c2)/denom;

	end
end

function DS_Distort:OnMouseDown(moho, mouseEvent)
moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
self.handle='none'
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	
	if self.initialized == false then
		self:getInitialValues(moho)
		self.initialized = true
	end
	
	
	--mouseEvent.view:DrawMe()
end

function DS_Distort:overlapCheck(vec1, vec2, sensitivity )
	if (vec1.x>vec2.x-sensitivity) and (vec1.x < vec2.x + sensitivity) and (vec1.y>vec2.y-sensitivity) and (vec1.y < vec2.y + sensitivity) then
		return true
	else
		return false
	end
end

function DS_Distort:OnMouseMoved(moho, mouseEvent)

local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	self:checkWhichHandle (moho, mouseEvent)
	
	if self.handle == 'topLeft' then
		self.topLeft.x = mouseEvent.vec.x
		self.topLeft.y = mouseEvent.vec.y
	elseif self.handle == 'topRight'  then
		self.topRight.x = mouseEvent.vec.x
		self.topRight.y = mouseEvent.vec.y
	elseif self.handle == 'bottomRight'  then
		self.bottomRight.x = mouseEvent.vec.x
		self.bottomRight.y = mouseEvent.vec.y
	elseif self.handle == 'bottomLeft'   then
		self.bottomLeft.x = mouseEvent.vec.x
		self.bottomLeft.y = mouseEvent.vec.y
	end
	
	self:distort(moho)
	mouseEvent.view:DrawMe()

end

function DS_Distort:OnMouseUp(moho, mouseEvent)
	self.handle = 'none'
	moho:AddPointKeyframe(moho.frame)
	moho:NewKeyframe(CHANNEL_POINT)
	mouseEvent.view:DrawMe()
	

end



function DS_Distort:checkWhichHandle (moho, mouseEvent)

local sensitivity = 0.075

	if self:overlapCheck(mouseEvent.vec, DS_Distort.bottomLeft, sensitivity) and self.handle == 'none' then
		self.handle = 'bottomLeft'
		return
	end
	if self:overlapCheck(mouseEvent.vec, DS_Distort.topLeft, sensitivity)  and self.handle == 'none' then
		self.handle = 'topLeft'
		return
	end
	if self:overlapCheck(mouseEvent.vec, DS_Distort.bottomRight, sensitivity)  and self.handle == 'none' then
		self.handle = 'bottomRight'
		return
	end
	if self:overlapCheck(mouseEvent.vec, DS_Distort.topRight, sensitivity)  and self.handle == 'none' then
		self.handle = 'topRight'
		return
	end
end

function DS_Distort:Keyed(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	local f=moho.layer:CurFrame()
	mesh:SelectNone()
		for i = 0, mesh:CountPoints() - 1 do
			if (mesh:Point(i).fAnimPos:HasKey(f)) then
				mesh:Point(i).fSelected = true
			end
		end

end

function DS_Distort:DrawMe(moho, view)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.numSel = moho:CountSelectedPoints()
	if (self.numSel < 2) then
		return
	end

	local g = view:Graphics()
	local min = LM.Vector2:new_local()
	local max = LM.Vector2:new_local()
	local markerR = 4
	local matrix = LM.Matrix:new_local()

	mesh:SelectedBounds(min, max)
	moho.layer:GetFullTransform(moho.frame, matrix, moho.document)

	g:Push()
	g:ApplyMatrix(matrix)

	g:SetColor(MOHO.MohoGlobals.SelCol) --(MOHO.MohoGlobals.SelCol)
	g:SetSmoothing(true)
	
	if not self.initialized then
		self.bottomLeft.x =min.x
		self.bottomLeft.y =min.y
		
		self.topLeft.x =min.x
		self.topLeft.y =max.y
		
		self.bottomRight.x =max.x
		self.bottomRight.y =min.y
		
		self.topRight.x =max.x
		self.topRight.y =max.y
	end

	--self.midPoint.x = (self.bottomLeft.x + self.topLeft.x + self.topRight.x + self.bottomRight.x) /4
	--self.midPoint.y = (self.bottomLeft.y + self.topLeft.y + self.topRight.y + self.bottomRight.y) /4

	g:DrawLine(self.bottomLeft.x, self.bottomLeft.y, self.topLeft.x, self.topLeft.y)
	g:DrawLine(self.topLeft.x, self.topLeft.y, self.topRight.x, self.topRight.y)
	g:DrawLine(self.topRight.x, self.topRight.y, self.bottomRight.x, self.bottomRight.y)
	g:DrawLine(self.bottomRight.x, self.bottomRight.y, self.bottomLeft.x, self.bottomLeft.y)
	g:SetSmoothing(false)
	
		
	g:DrawFatMarker(self.bottomLeft.x, self.bottomLeft.y, markerR) -- lower left corner
	g:DrawFatMarker(self.bottomRight.x, self.bottomRight.y, markerR) -- lower right corner
	g:DrawFatMarker(self.topLeft.x, self.topLeft.y, markerR) -- top left corner
	g:DrawFatMarker(self.topRight.x, self.topRight.y, markerR) -- top right corner
	
	
	--g:DrawFatMarker((min.x + max.x) / 2, min.y, markerR)
	--g:DrawFatMarker((min.x + max.x) / 2, max.y, markerR)
	--g:DrawFatMarker(min.x, (min.y + max.y) / 2, markerR)
	--g:DrawFatMarker(max.x, (min.y + max.y) / 2, markerR)

	g:Pop()
end


-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

DS_Distort.CHANGE = MOHO.MSG_BASE
DS_Distort.RESET = MOHO.MSG_BASE + 1
DS_Distort.FLIP_H = MOHO.MSG_BASE + 2
DS_Distort.FLIP_V = MOHO.MSG_BASE + 3
DS_Distort.SELKEYED = MOHO.MSG_BASE + 4

function DS_Distort:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/ScaleLayer/Scale=Scale")))

	layout:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/ScaleLayer/X=X:")))
	self.textX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textX)

	layout:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/ScaleLayer/Y=Y:")))
	self.textY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textY)

	layout:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/ScaleLayer/Z=Z:")))
	self.textZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textZ)

	layout:AddChild(LM.GUI.Button("Reset distortion points", self.RESET))
	
	layout:AddChild(LM.GUI.Button("Select keyed points", self.SELKEYED))

	layout:AddChild(LM.GUI.ImageButton("ScriptResources/flip_layer_h", MOHO.Localize("/Scripts/Tool/SetOrigin/FlipH=Flip Layer Horizontally"), false, self.FLIP_H))
	layout:AddChild(LM.GUI.ImageButton("ScriptResources/flip_layer_v", MOHO.Localize("/Scripts/Tool/SetOrigin/FlipV=Flip Layer Vertically"), false, self.FLIP_V))
end

function DS_Distort:UpdateWidgets(moho)
	self.textX:SetValue(moho.layer.fScale.value.x)
	self.textY:SetValue(moho.layer.fScale.value.y)
	self.textZ:SetValue(moho.layer.fScale.value.z)
end

function DS_Distort:HandleMessage(moho, view, msg)
	local newVal = LM.Vector3:new_local()
	local selCount = moho.document:CountSelectedLayers()

	if (msg == self.RESET) then
		

		
		self:getInitialValues(moho)
		self.initialized = true
		self:UpdateWidgets(moho)
	elseif (msg == self.CHANGE) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()

		newVal.x = self.textX:FloatValue()
		newVal.y = self.textY:FloatValue()
		newVal.z = self.textZ:FloatValue()
		for i = 0, selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			local vec = layer.fScale:GetValue(moho.frame + layer:TotalTimingOffset()) - newVal
			if (vec:Mag() > 0.0001) then
				layer.fScale:SetValue(moho.frame + layer:TotalTimingOffset(), newVal)
				moho:NewKeyframe(CHANNEL_LAYER_S)
			end
		end
	elseif (msg == self.FLIP_H) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()
		for i = 0, selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			layer.fFlipH:SetValue(moho.frame + layer:TotalTimingOffset(), not layer.fFlipH.value)
			moho:NewKeyframe(CHANNEL_LAYER_FLIP_H)
		end
	elseif (msg == self.FLIP_V) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()
		for i = 0, selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			layer.fFlipV:SetValue(moho.frame + layer:TotalTimingOffset(), not layer.fFlipV.value)
			moho:NewKeyframe(CHANNEL_LAYER_FLIP_V)
		end
	elseif (msg == self.SELKEYED) then
		self:Keyed(moho)
	end
	
end
